const t = require('tcomb')
const Gun = require('gun')
require('gun/lib/later')
require('gun/lib/then')
const {DateTime,} = require('luxon')

const {implementationError,} = require('../errors')

const DEFAULTS = {
  partition: 'cache',
}

exports = module.exports = (options) => {
  const settings = {
    ...DEFAULTS,
    ...(options || {}),
  }

  const state = {
    settings,
  }

  const start = () => {
    if (state.db == null) {
      state.db = Gun()
    }
  }

  const stop = () => {
    state.db = null
  }

  const get = async (id, logger) => {
    t.String(id, ['id',])
    t.Object(logger, ['logger',])

    if (state.db == null) {
      throw implementationError({message: 'Connection not started',})
    }

    return await state.db.get(state.settings.partition).get(id).then()
  }

  const set = async (id, item, ttl, logger) => {
    t.String(id, ['id',])
    t.String(item, ['item',])
    t.Number(ttl, ['ttl',])
    t.Object(logger, ['logger',])

    if (state.db == null) {
      throw implementationError({message: 'Connection not started',})
    }
    
    const now = DateTime.utc().toMillis()
    const path = `${settings.partition}/${id}`
    logger.debug(`Storing item at path ${path}`)
    await state.db.get(state.settings.partition).get(id).put({item, ttl, stored: now,})
      .later(() => {
        logger.debug(`Deleting envelope with key ${path} due to expiration`)
        state.db.get(path).put(null)
      }, ttl).then()
  }

  const drop = async (id, logger) => {
    t.Object(logger, ['logger',])
    
    if (state.db == null) {
      throw implementationError({message: 'Connection not started',})
    }

    await state.db.get(state.settings.partition).get(id).put(null).then()
  }

  return {
    start,
    stop,
    get,
    set,
    drop,
  }
}
